function [f, xopt,xhist,fhist] = goldensection(costf,x0)
% function [f, xopt,xhist,fhist] = goldensection(costf,x0)
%  perform a Golden Section search to minimize the function costf,
%  on the interval [0,infinity)
%  We assume that the function is decreasing at x=0 !!!!!!
%  if you have a good guess, give it as x0;
%  if you have a bracket, give it as x0 in the form
%  [xval1,xval2,fval1,fval2] or [xval1,xval2]
%  otherwise set x0 to the empty matrix: x0 = []
%  and this function will use a default of x0=1 and find a bracket

golden_section_tolerance = .01 ; %  1 percent=width of bracket/midpt of bracket

xhist = [];
fhist = [];

%  first, find a bracket if needed
if( length(x0) <= 1 )
  [xvals,fvals,xhist,fhist] = bracket(costf,x0);
  aval = xvals(1);
  cval = xvals(2);
  afval = fvals(1);
  cfval = fvals(2);
elseif( length(x0)==2 )
  xvals = x0;
  aval = xvals(1);
  cval = xvals(2);
  afval = costf(aval);
  cfval = costf(cval);
elseif( length(x0) == 4)
  xvals = x0(1:2);
  fvals = x0(3:4);
  aval = xvals(1);
  cval = xvals(2);
  afval = fvals(1);
  cfval = fvals(2);
end

%  Start a Golden Section search with
%  the initial interval [aval,cval]
%  our two midpoints will be m1 and m2, so aval < m1 < m2 < cval
GSinv = 2/(1+sqrt(5)); %  Golden Section inverse, about 0.618
GSinvc = 1-GSinv; %  complement of GSinv, about 0.382

m1 = (cval-aval)*GSinvc + aval;
m1f = costf(m1);
xhist(end+1) = m1;
fhist(end+1) = m1f;
m2 = (cval-aval)*GSinv  + aval;
m2f = costf(m2);
xhist(end+1) = m2;
fhist(end+1) = m2f;


while( (cval-aval)/((aval+cval)/2) > golden_section_tolerance )
  if( m1f < m2f ) %  then our interval shrinks toward the left
%     disp('going left')
    cval=m2;
    cfval=m2f;
    m2=m1;
    m2f=m1f;
    m1 = (cval-aval)*GSinvc + aval;
    m1f = costf(m1);
    xhist(end+1) = m1;
    fhist(end+1) = m1f;
else %  m1f > m2f, so our interval shrinks toward the right
%       disp('going right')
    aval=m1;
    afval=m1f;
    m1=m2;
    m1f=m2f;
    m2 = (cval-aval)*GSinv  + aval;
    m2f = costf(m2);
    xhist(end+1) = m2;
    fhist(end+1) = m2f;
  end %  if
% disp([aval,m1,m2,cval])
end %  while

%  What single value should we return?
%  we could pick the midpoint of our bracket
%  xopt = (cval+aval)/2;
%  f = costf(xopt);
%      xhist(end+1) = xopt;
%     fhist(end+1) = f;

%  or we could take the minimum of our search history
[f, indx ] = min(fhist);
xopt = xhist(indx);

%  Try this to run it:
%  deff('[y]=myparab(x)','y=(x-10)^2;');     
%  [f, xopt,xhist,fhist] = goldensection(myparab,1)
%  or
%  [f, xopt,xhist,fhist] = goldensection(myparab,3)
%  plot(xhist,fhist,'.-');

%  or, to set up a line-search sort of thing,
%  Qmat=[1 0;0 2]; xvec=[1;1];dvec=-2*Qmat*xvec; %  dvec=-gradient
%  deff('[y]=linesearchhelper(alpha)','y=purequad(xvec+alpha*dvec,2);');
%  [f, xopt,xhist,fhist] = goldensection(linesearchhelper,1)
%  plot(xhist,fhist,'.-');

